""" A mention, which is used in the Redcoat documents."""


class Mention(object):
    """A single entity mention. Captures the start, end, tokens and
    labels of the mention."""

    def __init__(
        self,
        start: int,
        end: int,
        tokens: list,
        labels: list,
        mention_id: int,
    ):
        super(Mention, self).__init__()
        self.start = start
        self.end = end
        self.tokens = tokens
        self.labels = labels
        self.mention_id = mention_id

    def get_first_label(self):
        """A simple function to retrieve the first label of a mention.

        Args:
            mention (dict): The mention to get the first label of.

        Returns:
            str: The first label, e.g. "Item".
        """
        labels = self.labels
        all_labels = [x for x in labels]
        first_label = all_labels[0] if len(all_labels) > 0 else None
        return first_label

    def to_dict(self):
        return self.__dict__

    def __repr__(self):
        if len(self.labels) == 1:
            return f"({' '.join(self.tokens)} [{self.labels[0]}])"
        else:
            return f"({' '.join(self.tokens)} {self.labels})"



""" A relation, which is used in the Redcoat documents."""
from typing import List

from .Mention import Mention


class Relation(object):
    """A single relation. Captures the start mention, end mention, tokens and
    labels of the relation."""

    def __init__(self, start: Mention, end: Mention, label: str):
        super(Relation, self).__init__()

        if start == end:
            raise ValueError(
                "Cannot create relation between a mention and itself"
            )

        self.start = start
        self.end = end
        self.label = label

    def to_dict(self):
        """Return a dictionary representation of this Relation.
        Convert 'start' and 'end' to the mention index of this
        relation's start and end mention.

        Returns:
            dict: Dictionary representation of this Relation.
        """
        return {
            "start": self.start.mention_id,
            "end": self.end.mention_id,
            "type": self.label,
        }

    def __repr__(self):
        return (
            f"({' '.join(self.start.tokens)})-"
            f"[{self.label}]->({' '.join(self.end.tokens)})"
        )
